package com.example.test;

import static org.junit.Assert.assertEquals;

import java.io.StringReader;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.junit.Before;
import org.junit.Test;
import org.xml.sax.SAXException;

public class FooTypeTest {
	private Schema schema;
	
	@Before
	public void setUp() throws SAXException {
		SchemaFactory sf = SchemaFactory.newInstance(javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI);
		schema = sf.newSchema(getClass().getResource("/test.xsd"));
	}
	
	private FooType unmarshalFoo(Unmarshaller u, String xml) throws JAXBException {
		JAXBElement<FooType> e = u.unmarshal(
				new StreamSource(new StringReader(xml)), 
				FooType.class);
		return e.getValue();
	}
	
	@Test
	public void testNonValidatingExplicitValidValue() throws JAXBException {
		JAXBContext ctx = JAXBContext.newInstance(FooType.class);
		Unmarshaller u = ctx.createUnmarshaller();
		String xml = "<foo xmlns='http://www.example.com/test' enum='enumB'/>";
		FooType foo = unmarshalFoo(u, xml);
		assertEquals(EnumType.ENUM_B, foo.getEnum());
	}
	
	@Test
	public void testNonValidatingDefaultValue() throws JAXBException {
		JAXBContext ctx = JAXBContext.newInstance(FooType.class);
		Unmarshaller u = ctx.createUnmarshaller();
		String xml = "<foo xmlns='http://www.example.com/test'/>";
		FooType foo = unmarshalFoo(u, xml);
		assertEquals(EnumType.ENUM_A, foo.getEnum());
	}
	
	@Test(expected=UnmarshalException.class)
	public void failingTestNonValidatingInvalidValue() throws JAXBException {
		JAXBContext ctx = JAXBContext.newInstance(FooType.class);
		Unmarshaller u = ctx.createUnmarshaller();
		String xml = "<foo xmlns='http://www.example.com/test' enum='invalid'/>";
		// The next call will not throw an exception, but instead will not set any value for the 'enum'
		// on the resulting foo, so accessing foo.getEnum() will return the default.
		FooType foo = unmarshalFoo(u, xml);
	}
	
	@Test
	public void testValidatingExplicitValidValue() throws JAXBException {
		JAXBContext ctx = JAXBContext.newInstance(FooType.class);
		Unmarshaller u = ctx.createUnmarshaller();
		u.setSchema(schema);
		String xml = "<foo xmlns='http://www.example.com/test' enum='enumB'/>";
		FooType foo = unmarshalFoo(u, xml);
		assertEquals(EnumType.ENUM_B, foo.getEnum());
	}
	
	@Test
	public void testValidatingDefaultValue() throws JAXBException {
		JAXBContext ctx = JAXBContext.newInstance(FooType.class);
		Unmarshaller u = ctx.createUnmarshaller();
		u.setSchema(schema);
		String xml = "<foo xmlns='http://www.example.com/test'/>";
		FooType foo = unmarshalFoo(u, xml);
		assertEquals(EnumType.ENUM_A, foo.getEnum());
	}
	
	@Test(expected=UnmarshalException.class)
	public void testValidatingInvalidValue() throws JAXBException {
		JAXBContext ctx = JAXBContext.newInstance(FooType.class);
		Unmarshaller u = ctx.createUnmarshaller();
		u.setSchema(schema);
		String xml = "<foo xmlns='http://www.example.com/test' enum='invalid'/>";
		
		// Will throw UnmarshalException with 
		// [org.xml.sax.SAXParseException: cvc-enumeration-valid: Value 'invalid' is not facet-valid with respect to enumeration '[enumA, enumB]'. It must be a value from the enumeration.]
		/* ignored = */ unmarshalFoo(u, xml);
	}
}
